$INTERFACE U1FLU1                                             97/12/01
$REM LU
  $SET(DINIT)
  $ENDSET
  $N='NF'
  $ADD(INTEGER,'\IMD')
  $IF(CONSTR='S')
    $CONSTR='C'
    $ADD(REAL,'\UXUDOT')
  $ELSE
    $ADD(REAL,'\UXVDOT')
  $ENDIF
  $ISNF=0
  $TYPE='L'
  $UPDATE='NO'
  $DEFAULT='V'
  $IF($DEF(KCF))
    $IF(KCF=1) $DEFAULT='C'
  $ENDIF
  $IF($DEF(KCA))
    $IF(KCA=1) $DEFAULT='C'
  $ENDIF
  $HELP
    FORM OF METHOD:
      C - CONJUGATE GRADIENT METHODS
      V - VARIABLE METRIC METHODS WITH LIMITED
          STORAGE
  $CHECK(DECOMP,DEFAULT,'C\V',2,NO)
  $IF(DECOMP='C')
    $HELP
      METHOD
        1 - BASIC CONJUGATE GRADIENT METHODS
        2 - STOREY MODIFIED CONJUGATE GRADIENT METHODS
    $CHECK(NUMBER,1,'1\2\3\4',2,NO)
    $IF(NUMBER=2) $ITRG=1
  $ELSE
    $HELP
      METHOD
        1 - CLASSIC LIMITED MEMORY BFGS METHOD WITH THE STRANG
            RECURRENCES
        2 - METHOD OF HU AND STOREY
        3 - TRANSFORMATION OF THE LIMITED MEMORY BROYDEN CLASS
            TO THE BFGS METHOD WITH THE STRANG RECURRENCES
        4 - SIMPLIFIED TRANSFORMATION OF THE LIMITED MEMORY
            BROYDEN CLASS TO THE BFGS METHOD WITH THE STRANG
            RECURRENCES
        5 - THE BFGS METHOD THAT USES THE PREVIOUS DIFFERENCES
        6 - THE BFGS METHOD THAT USES CONJUGATE DIFFERENCES
        7 - THE BFGS METHOD THAT USES CORRECTED DIFFERENCES
        8 - THE BLOCK BFGS METHOD
        9 - THE BNS METHOD
    $CHECK(NUMBER,1,'1\2\3\4\5\6\7\8\9',2,NO)
    $IF(NUMBER=1)
      $HELP
        SELECTION OF THE SELF SCALING
          1 - SELF SCALING SUPRESSED
          2 - SCALAR SELF SCALING
          3 - DIAGONAL SELF SCALING
          4 - SCALAR AND DIAGONAL SELF SCALING
          5 - STEIGAUG NEWTON TYPE SCALING
      $CHECK(MET1,2,'1\2\3\4\5',4)
      $IF(MET1=5) $ITRG=1
    $ENDIF
  $ENDIF
  $HELP
    TOLERANCE FOR DESCENT DIRECTION
  $CHECK(EPS0,'1.0$P-8',REAL,4)
  $HELP
    SPECIFICATION OF THE SEARCH:
      B - BASIC LINE SEARCH
      H - HAGER LINE SEARCH
      M - LINE SEARCH WITH TRUST REGION BOUND
      N - NONMONOTONE LINE SEARCH
  $CHECK(SEARCH,'B','B\H\M\N',3,NO)
  $IF(SEARCH='H')
    $HELP
      MAXIMUM NUMBER OF TESTS ON STEPLENGTH
    $CHECK(MTESX,5,INT,4)
    $HELP
      MAXIMUM NUMBER OF TESTS ON FUNCTION DECREASE
    $CHECK(MTESF,5,INT,4)
    $HELP
      LOWER BOUND FOR STEPLENGTH
    $CHECK(TOLX,'1.0$P-18',REAL,3)
    $HELP
      LOWER BOUND FOR FUNCTION DECREASE
    $CHECK(TOLF,'1.0$P-18',REAL,3)
    $INITR=0
  $ELSE
    $DEFAULT=-3
    $IF(DECOMP='V') $DEFAULT=2
    $HELP
      CHOICE OF THE INITIAL STEPSIZE:
               0 - INITIAL STEPSIZE IS SPECIFIED IN THE CALLING PROGRAM
         -1 OR 1 - UNIT INITIAL STEPSIZE
         -2 OR 2 - COMBINED UNIT AND QUADRATICALLY ESTIMATED INITIAL
                   STEPSIZE (DAVIDON)
         -3 OR 3 - COMBINED UNIT AND QUADRATICALLY ESTIMATED INITIAL
                   STEPSIZE (STOREY )
         -4 OR 4 - QUADRATICALLY ESTIMATED INITIAL STEPSIZE
       INITS < 0 - THE PREVIOUS FUNCTION VALUE IS USED
       INITS > 0 - THE MINIMUM  FUNCTION VALUE IS USED
    $CHECK(INITS,DEFAULT,'-4\-3\-2\-1\0\1\2\3\4',4)
    $HELP
      INITIAL STEPSIZE IN THE FIRST ITERATION:
        0 - STANDARD CHOICE IN THE FIRST ITERATION
        1 - NUMERICAL DIFFERENTIATION IN THE FIRST ITERATION
        2 - UNIT INITIAL STEPSIZE
    $CHECK(INITR,0,'0\1\2',4)
  $ENDIF
  $SUBST('UO0FU1')
  $SUBST('UZLIND')
  $IF(NAL>0)
    $MAL='$NAL'
    $ADD(INIT)
      MAL=NAL
    $ENDADD
    $JOB=1
    $IF(NORMA>0) $SUBST('UANRL3')
  $ENDIF
  $IF(CONSTR='C') $SUBST('UYFPC1')
  IF (ITERM.NE.0) GOTO $90
*
*     ---------------------------
*     CONJUGATE DIRECTION METHODS
*     TEMPLATE : U1FLU1
*     ---------------------------
*
  $LMD=$10
  ASSIGN $20 TO IMD
  CALL UYPRO1('UXFU',1)
  CALL UYPRO2(FMIN,FO)
  $IF(INITR>0)
    $ADD(INTEGER,'\INIST')
    INIST=INITS
  $ENDIF
$L=$10 ; $LABEL
  $KD=1
  $SUBST('UZMODN')
  $IF(KDS<1)
  $ELSEIF(CONSTR='C')
    IF(TSS(NS).EQ.'USXX') P=UXUDOT(NF,$G,$S,$IX,KBF)
  $ELSE
    IF(TSS(NS).EQ.'USXX') P=UXVDOT(NF,$G,$S)
  $ENDIF
  GO TO IMD
$L=$20 ; $LABEL
  $SUBST('UYTR$(CONSTR)G')
  $SUBST('UO2FU3')
  $SUBST('UYFUT1')
  IF (ITERM.NE.0) GOTO $90
  $IF(CONSTR='C')
    IF (RMAX.GT.ZERO) THEN
    $SUBST('UYRM$(CONSTR)0')
    ENDIF
  $ENDIF
$L=$30 ; $LABEL
  $IF(KDF=0)
    $IF(MCG<>0)
      $IF(MCG1=1)
        $SUBST('UYCP$(CON)S')
      $ENDIF
    $ENDIF
  $ENDIF
  $MAT='V'
*
*     -----------------------
*     DIRECTION DETERMINATION
*     TEMPLATE : UD$(MAT)$(TYPE)$(DECOMP)$(NUMBER)
*     -----------------------
*
  $SUBST('UD$(MAT)$(TYPE)$(DECOMP)$(NUMBER)')
*
*     ------------------------------
*     END OF DIRECTION DETERMINATION
*     ------------------------------
*
$REM  $IF(NUMBER=3)
$REM    CALL UYSET2
$REM    ASSIGN $70 TO IMD
$REM    GO TO $10
$REM  $L=$70
$REM  $ELSE
  $SUBST('UD1T$(TYPE)1')
  IF (ITERM.NE.0) GOTO $90
  IF (IREST.NE.0) GOTO $30
$L=$60 ; $LABEL
  $SUBST('UYTR$(CONSTR)S')
  $IF(CONSTR='C')
    IF (RMAX.EQ.ZERO) GO TO $75
  $ENDIF
  $IF(INITR>0)
    INITS=INIST
    IF (NIT.GT.KIT) GO TO $70
    IF (INITR.EQ.0) GO TO $70
    INITS=0
    $IF(INITR=1)
      $GA=GN
      PP=SQRT(ETA0)/SNORM
      LD=MIN(MAX(KDF,KDA)-1,0)
      $IF(MODEL='AF')
        LDA=LD
      $ELSE
        LDA=-1
      $ENDIF
      I=ISNA
      ISNA=0
      $SUBST('UYTR$(CONSTR)X')
      ASSIGN $65 TO IMD
      GO TO $LMD
$65   CONTINUE
      ISNA=I
      $SUBST('UYTR$(CONSTR)A')
      R=-PP*PO/UXVDDT($N,$SN,$GN,$GO)
    $ELSE
      R=ONE
    $ENDIF
  $ENDIF
$L=$70 ; $LABEL
  $SUBST('UZLINS')
  $IF(SEARCH='H')
    GO TO ($78,$72,$74) ISB+1
$L=$72 ; $LABEL
    ASSIGN $70 TO IMD
    GO TO $10
$L=$74 ; $LABEL
    $SUBST('US1LUP')
    GO TO ($70,$76) ISB+1
$L=$76 ; $LABEL
    ASSIGN $74 TO IMD
    GOTO $10
  $ELSE
    GO TO ($78,$72) ISB+1
$L=$72 ; $LABEL
    ASSIGN $70 TO IMD
    $IF(CONSTR='C')
      $IF(TYPE='G')
        CALL UXVDIR(NF,R,$S,$XO,$X)
      $ELSE
        CALL UXUDIR(NF,R,$S,$XO,$X,$IX,KBF)
      $ENDIF
      $SUBST('UCBS04')
    $ELSE
      CALL UXVDIR(NF,R,$S,$XO,$X)
    $ENDIF
    GOTO $10
  $ENDIF
$L=$78 ; $LABEL
  IF (ITERS.LE.0) THEN
  $IF(KDS<1)
    $SUBST('UYZER0')
  $ELSE
    $SUBST('UYZER1')
  $ENDIF
  IF(IDIR.EQ.0) THEN
  CALL UYRES1(TSXX)
  CALL UYSET1
  GO TO $30
  ELSE IF (MOT3.EQ.0) THEN
  CALL UYSET1
  GO TO $30
  ELSE
  ITERD=0
  ENDIF
  ENDIF
$REM  $ENDIF
  IF(KD.GT.LD) THEN
  ASSIGN $80 TO IMD
  GO TO $10
  ENDIF
$L=$80 ; $LABEL
  TXFU=TUXX
  $IF(KDF=0)
    $IF(MCG<>0)
      $IF(MCG1=1)
        $SUBST('UYUP$(CON)D')
      $ENDIF
    $ENDIF
  $ENDIF
$REM  $IF(MET<0)
$REM    $SUBST('UYTR$(CONSTR)C')
$REM  $ELSE
$REM    $SUBST('UYTR$(CONSTR)D')
$REM  $ENDIF
  $IF(NUMBER=3)
    CALL UXVSCL(NF,-R,$GO,$XS)
  $ENDIF
  $SUBST('UYTR$(CONSTR)D')
  $IF(NUMBER>2)
    $SUBST('UUVLV$NUMBER')
    IF (ITERH.NE.0) CALL UYRES1('UPDATE  ')
  $ENDIF
  $IF(CONSTR='C')
$L=$75 ; $LABEL
    $SUBST('UYADC0')
  $ENDIF
  GOTO $20
$L=$90 ; $LABEL
  IF(ITERM.LT.0) TXFU=TDXX
  CALL UYEPI1(1)
  $IF(NAL>0)
    $JOB=-1
    $IF(NORMA>0) $SUBST('UANRL3')
  $ENDIF
  $IF(INITR>0)
    INITS=INIST
  $ENDIF
  $SUBST('UO1FU2')
  $ADD(INIT)
    NXFU='U1FLU1'
  $ENDADD
  $EXIT
